==> MLInterfaces_1.28.0.tar.gz: MLInterfaces/R/MLearn.R <==
setGeneric("MLearn", function(formula, data, .method, trainInd, ...)standardGeneric("MLearn"))


setMethod("MLearn", c("formula", "data.frame", "learnerSchema",
   "numeric" ), function( formula, data, .method, trainInd, ...) {
## find software
  pname = .method@packageName
  fname = .method@mlFunName
## create the requested function
  lfun = do.call("::", list(pname, fname))
## build test and train subsets
  tedata = data[-trainInd,]
  trdata = data[trainInd,]
## execute on training data 
  ans = lfun( formula, trdata, ...)
## collect response subsets
  trFrame = try(model.frame(formula, trdata, na.action=na.fail))
  if (inherits(trFrame, "try-error")) stop("NA encountered in data.  Please rectify.")
  teFrame = model.frame(formula, tedata, na.action=na.fail)
  if (inherits(teFrame, "try-error")) stop("NA encountered in data.  Please rectify.")
  trout = model.response( trFrame )
  teout = model.response( teFrame )
## tell what was done
  thecall = match.call()
## convert the execute result into an MLint output container
  tmp = .method@converter( ans, data, trainInd )
## add some stuff to the converted representation
  if (!tmp@embeddedCV) {
      tmp@testOutcomes = teout
      tmp@trainOutcomes = trout
  }
  else tmp@testOutcomes = trout # if CV is embedded, the 'training' is 'test'
  tmp@call = thecall
  tmp@learnerSchema = .method
  tmp
})

es2df = function(x,keep=NULL) {
#
# the keep parameter says which pData vars are kept in
#
   if (is.null(keep)) return(data.frame(t(exprs(x)),pData(x)))
   else {
        tmp = data.frame(t(exprs(x)),pData(x)[[keep]])
        names(tmp)[ncol(tmp)] = keep
        return(tmp)
        }
}

setMethod("MLearn", c("formula", "ExpressionSet", "learnerSchema", "numeric" ),
  function(formula, data, .method, trainInd, ...) {
#
# the keep setting below says just keep the response variable
# from pData
#
        data = es2df(data, keep=as.character(as.list(formula)[[2]]))
	thecall = match.call()
        ans = MLearn( formula, data, .method, trainInd, ... )
 	ans@call = thecall
        ans@learnerSchema = .method
	ans
 })


# this .method for MLearn is devoted essentially to cross-validation.  it structures
# a series of calls to MLearn[numeric trainInd] and collects the output, suitably
# ordered, into a classifierOutput structure, in contrast to the older xvalML

# it is an open question whether we should try to keep all the RObjects generated through
# the sequence of cross-validations.  i think we can as long as we are not in LOO

setMethod("MLearn", c("formula", "data.frame", "learnerSchema",
   "xvalSpec" ), function( formula, data, .method, trainInd, ...) {
   xvspec = trainInd # rationalize parameter name
   xvalMethod = xvspec@type
   if (!(xvspec@type %in% c("LOO", "LOG", "NOTEST"))) stop("only supporting NOTEST (fit to all data), or LOO or LOG type xvalidation at this time")
   if (xvspec@type == "LOG" && is(xvspec@partitionFunc, "NULL")) stop("for xval type LOG, must supply partition function")
   thecall = match.call()
   tef = model.frame(formula, data)
   teo = model.response( tef )
   classLab = names(tef)[ respind <- attr( terms(formula,data=data), "response" ) ]

   N <- nrow(data)
   inds <- 1:N

   if (xvspec@type == "NOTEST")
        {
        ans = MLearn(formula, data, .method, 1:N, ...)
	ans@call = thecall
        return(ans)
        }

# deal with sample selection

   if (xvalMethod == "LOO")
     {
     n <- length(inds)
     selnProc <- function(i) -i   # how to get the training set from inds
     }
   else                          # FUN
     {
     n <- xvspec@niter
     selnProc <- function(i) xvspec@partitionFunc( data, classLab, i )  # func defines training set directly
     }

# deal with feature selection

   # check the supplied fsFun
        do.fs = FALSE
   	if (is.function(xvspec@fsFun)) {
	        do.fs = TRUE
		fsFun = xvspec@fsFun
		if (!all(names(formals(fsFun)) %in% c("formula", "data"))) {
			stop("xvspec@fsFun must have formals formula, data")
			}
		tst = fsFun(formula,data)
		if (!is( tst, "formula") ) {
			print("problem with fsFun in xvalSpec:")
			print(tst)
			stop("fsFun must return a formula; instead returned the object just printed.")
		}
	}

   xvalidator <- function(i, ...) {
     idx <- selnProc(i) # need to change sign when reordering...
     if (do.fs) fmla2use=fsFun(formula, data[inds[idx],])  # we are clobbering input formula
      else fmla2use=formula
     rhs_fmla = function (f) colnames(attr(terms(f, data=data), "factors"))
     list( test.idx=(setdiff(inds,idx)), mlans=MLearn( fmla2use, data, .method=.method, trainInd=inds[idx], ...),
             featInUse= rhs_fmla(fmla2use) ) # package result -- test.idx kept for rearrangement
     }

#   xvalLoop = xvalLoop(NULL) # eventually will allow clusters

   out <- lapply( 1:n, xvalidator, ... )
   classif <- unlist( sapply( out, function(x) testPredictions(x[["mlans"]]) ) )
# now want the test sets for the various iterations
   ords <- unlist( lapply( out, function(x) x[["test.idx"]] ) )
   featsUsed = list()
   if (do.fs) featsUsed = lapply(  out, function(x) x[["featInUse"]] )
   reord = match(inds, ords)
   testpred = factor(classif)[reord]
   new("classifierOutput", testPredictions=factor(testpred), testOutcomes=teo, call=thecall, RObject = out, fsHistory=featsUsed, learnerSchema=.method)
})


setMethod("MLearn", c("formula", "ExpressionSet", "learnerSchema",
   "xvalSpec" ), function( formula, data, .method, trainInd, ...) {
	thecall = match.call()
        data = es2df(data, keep=as.character(as.list(formula)[[2]]))
	ans = MLearn(formula, data, .method, trainInd, ...)
	ans@call = thecall
        ans@learnerSchema = .method
	ans
})

==> MLInterfaces_1.30.0.tar.gz: MLInterfaces/R/MLearn.R <==
setGeneric("MLearn", function(formula, data, .method, trainInd, ...)standardGeneric("MLearn"))


setMethod("MLearn", c("formula", "data.frame", "learnerSchema",
   "numeric" ), function( formula, data, .method, trainInd, ...) {
## find software
  pname = .method@packageName
  fname = .method@mlFunName
## create the requested function
  lfun = do.call("::", list(pname, fname))
## build test and train subsets
  tedata = data[-trainInd,]
  trdata = data[trainInd,]
## execute on training data 
  ans = lfun( formula, trdata, ...)
## collect response subsets
  trFrame = try(model.frame(formula, trdata, na.action=na.fail))
  if (inherits(trFrame, "try-error")) stop("NA encountered in data.  Please rectify.")
  teFrame = model.frame(formula, tedata, na.action=na.fail)
  if (inherits(teFrame, "try-error")) stop("NA encountered in data.  Please rectify.")
  trout = model.response( trFrame )
  teout = model.response( teFrame )
## tell what was done
  thecall = match.call()
## convert the execute result into an MLint output container
  tmp = .method@converter( ans, data, trainInd )
## add some stuff to the converted representation
  if (!tmp@embeddedCV) {
      tmp@testOutcomes = teout
      tmp@trainOutcomes = trout
  }
  else tmp@testOutcomes = trout # if CV is embedded, the 'training' is 'test'
  tmp@call = thecall
  tmp@learnerSchema = .method
  tmp
})

es2df = function(x,keep=NULL) {
#
# the keep parameter says which pData vars are kept in
#
   if (is.null(keep)) return(data.frame(t(exprs(x)),pData(x)))
   else {
        tmp = data.frame(t(exprs(x)),pData(x)[[keep]])
        names(tmp)[ncol(tmp)] = keep
        return(tmp)
        }
}

setMethod("MLearn", c("formula", "ExpressionSet", "learnerSchema", "numeric" ),
  function(formula, data, .method, trainInd, ...) {
#
# the keep setting below says just keep the response variable
# from pData
#
        data = es2df(data, keep=as.character(as.list(formula)[[2]]))
	thecall = match.call()
        ans = MLearn( formula, data, .method, trainInd, ... )
 	ans@call = thecall
        ans@learnerSchema = .method
	ans
 })


# this .method for MLearn is devoted essentially to cross-validation.  it structures
# a series of calls to MLearn[numeric trainInd] and collects the output, suitably
# ordered, into a classifierOutput structure, in contrast to the older xvalML

# it is an open question whether we should try to keep all the RObjects generated through
# the sequence of cross-validations.  i think we can as long as we are not in LOO

setMethod("MLearn", c("formula", "data.frame", "learnerSchema",
   "xvalSpec" ), function( formula, data, .method, trainInd, ...) {
   xvspec = trainInd # rationalize parameter name
   xvalMethod = xvspec@type
   if (!(xvspec@type %in% c("LOO", "LOG", "NOTEST"))) stop("only supporting NOTEST (fit to all data), or LOO or LOG type xvalidation at this time")
   if (xvspec@type == "LOG" && is(xvspec@partitionFunc, "NULL")) stop("for xval type LOG, must supply partition function")
   thecall = match.call()
   tef = model.frame(formula, data)
   teo = model.response( tef )
   classLab = names(tef)[ respind <- attr( terms(formula,data=data), "response" ) ]

   N <- nrow(data)
   inds <- 1:N

   if (xvspec@type == "NOTEST")
        {
        ans = MLearn(formula, data, .method, 1:N, ...)
	ans@call = thecall
        return(ans)
        }

# deal with sample selection

   if (xvalMethod == "LOO")
     {
     n <- length(inds)
     selnProc <- function(i) -i   # how to get the training set from inds
     }
   else                          # FUN
     {
     n <- xvspec@niter
     selnProc <- function(i) xvspec@partitionFunc( data, classLab, i )  # func defines training set directly
     }

# deal with feature selection

   # check the supplied fsFun
        do.fs = FALSE
   	if (is.function(xvspec@fsFun)) {
	        do.fs = TRUE
		fsFun = xvspec@fsFun
		if (!all(names(formals(fsFun)) %in% c("formula", "data"))) {
			stop("xvspec@fsFun must have formals formula, data")
			}
		tst = fsFun(formula,data)
		if (!is( tst, "formula") ) {
			print("problem with fsFun in xvalSpec:")
			print(tst)
			stop("fsFun must return a formula; instead returned the object just printed.")
		}
	}

   xvalidator <- function(i, ...) {
     idx <- selnProc(i) # need to change sign when reordering...
     if (do.fs) fmla2use=fsFun(formula, data[inds[idx],])  # we are clobbering input formula
      else fmla2use=formula
     rhs_fmla = function (f) colnames(attr(terms(f, data=data), "factors"))
     list( test.idx=(setdiff(inds,idx)), mlans=MLearn( fmla2use, data, .method=.method, trainInd=inds[idx], ...),
             featInUse= rhs_fmla(fmla2use) ) # package result -- test.idx kept for rearrangement
     }

#   xvalLoop = xvalLoop(NULL) # eventually will allow clusters

   out <-
       if (is.loaded("mc_fork", PACKAGE="multicore")) {
           mcLapply <- get("mclapply", envir=getNamespace("multicore"))
           mcLapply(1:n, xvalidator, ...)
       } else {
           lapply( 1:n, xvalidator, ... )
       }  # thanks Martin Morgan!
   classif <- unlist( sapply( out, function(x) testPredictions(x[["mlans"]]) ) )
 # now want the test sets for the various iterations
   ords <- unlist( lapply( out, function(x) x[["test.idx"]] ) )

   classif <- unlist( sapply( out, function(x) testPredictions(x[["mlans"]]) ) )
# now want the test sets for the various iterations
   ords <- unlist( lapply( out, function(x) x[["test.idx"]] ) )
   featsUsed = list()
   if (do.fs) featsUsed = lapply(  out, function(x) x[["featInUse"]] )
   reord = match(inds, ords)
   testpred = factor(classif)[reord]
   new("classifierOutput", testPredictions=factor(testpred), testOutcomes=teo, call=thecall, RObject = out, fsHistory=featsUsed, learnerSchema=.method)
})


setMethod("MLearn", c("formula", "ExpressionSet", "learnerSchema",
   "xvalSpec" ), function( formula, data, .method, trainInd, ...) {
	thecall = match.call()
        data = es2df(data, keep=as.character(as.list(formula)[[2]]))
	ans = MLearn(formula, data, .method, trainInd, ...)
	ans@call = thecall
        ans@learnerSchema = .method
	ans
})
